@@ -409,15 +409,34 @@ void playwav(int thefd, char hd_buf[20], int mods, int speed, int bits, int ster
 
     memcpy((void*)&wavhd, (void*)hd_buf, 20);
     count = read(thefd, ((char*)&wavhd)+20, sizeof(wavhd) - 20);
+
+#if  __BYTE_ORDER == __BIG_ENDIAN
+#include<byteswap.h>
+    /* let's do a bit of reordering */
+    wavhd.length =  bswap_32 (wavhd.length);
+    wavhd.sc_len =  bswap_32 (wavhd.sc_len);
+    wavhd.format =  bswap_16 (wavhd.format);
+    wavhd.modus  =  bswap_16 (wavhd.modus);
+    
+    wavhd.sample_fq  =  bswap_32 (wavhd.sample_fq);
+    wavhd.byte_p_sec =  bswap_32 (wavhd.byte_p_sec);
+    
+    wavhd.byte_p_spl =  bswap_16 (wavhd.byte_p_spl);
+    wavhd.bit_p_spl  =  bswap_16 (wavhd.bit_p_spl);
+    
+    wavhd.data_chunk =  bswap_32 (wavhd.data_chunk);
+    wavhd.data_length =  bswap_32 (wavhd.data_length);
+#endif
+   
     if(wavhd.format != 1) Die("input is not a PCM WAV file");
     if (! (mods&MSPEED))
-	speed = wavhd.sample_fq;
+      speed = wavhd.sample_fq;
     if (! (mods&MBITS))
-	bits = wavhd.bit_p_spl;
+      bits = wavhd.bit_p_spl;
     if (! (mods&MSTEREO))
-	stereo = wavhd.modus - 1;
-    fprintf(stderr, "Playing WAVE : %d bit, Speed %d %s ...\n",
-	bits, speed, (stereo)? "Stereo" : "Mono");
+      stereo = wavhd.modus - 1;
+    fprintf(stderr, "Playing WAVE : %d bit, Speed %d %s ...\n", 
+	    bits, speed, (stereo)? "Stereo" : "Mono");
     diskread(thefd, 0, NULL, 1, speed, bits, stereo);
 }
 
