@@ -4,12 +4,15 @@
 **
 ** Copyright under the GPL - see the file COPYING in this directory
 **
+** 1999.02.09. huba
+** Fixed a bug in semaphore handling
 **
 */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <string.h>
 #include <errno.h>
 #include <signal.h>
 #include <sys/types.h>
@@ -18,9 +21,16 @@
 #include <sys/shm.h>
 #include <sys/wait.h>
 
-/* The size of the big array */
+#ifdef __GLIBC__
+/* Nasty hack */
+#ifndef SEMMSL
+#define SEMMSL 32
+#endif
+#endif
+
+/* The default size of the big array */
 /* (currently 256K - nearly 1.5 sec at max rate) */
-#define BIGBUFFSIZE 0x040000
+#define DEFAULTBUFFSIZE 0x040000
 
 /* Types */
 typedef struct blockinf_t
@@ -33,6 +43,19 @@ typedef struct blockinf_t
     int stereo;
 } blockinf_t;
 
+#ifdef _SEM_SEMUN_UNDEFINED		/* is semun defined in <sys/sem.h>? */
+union semun
+{
+	int val;			/* value for SETVAL */
+	struct semid_ds *buf;		/* buffer for IPC_STAT & IPC_SET */
+	unsigned short int *array;	/* array for GETALL & SETALL */
+	struct seminfo *__buf;		/* buffer for IPC_INFO */
+};
+#endif 
+
+/* Globals */
+int bigbuffsize = DEFAULTBUFFSIZE;
+
 /* Statics - mostly shared memory etc */
 static int shmid, shmid2, *disksemid, *sndsemid;
 static char *bigbuff;
@@ -48,19 +71,27 @@ static void sighandler(int i);
 extern int abuf_size;
 extern int audio;
 extern char *progname;
-
+extern pid_t pid;
+extern int recorder;
+extern int debug;
 
 /* extern prototypes */
 extern void ErrDie(char *err);
 extern void snd_parm(int speed, int bits, int stereo);
 extern void sync_audio(void);
+extern void cleanup_audio(void);
 
 void init_shm(void)
 {
     int i;
 
+    /* Round up to a multiple of abuf_size */
+    if (bigbuffsize % abuf_size != 0) {
+	    bigbuffsize = ((bigbuffsize / abuf_size) +1) * abuf_size;
+    }
+
 	/* Create, attach and mark for death the big buffer */
-    shmid = shmget(IPC_PRIVATE, BIGBUFFSIZE,
+    shmid = shmget(IPC_PRIVATE, bigbuffsize,
 	IPC_EXCL | IPC_CREAT | 0600);
     if (shmid == -1)
 	ErrDie("shmget");
@@ -78,7 +109,7 @@ void init_shm(void)
     /* Create an array of pointers. Point them at equally spaced
     ** chunks in the main buffer, to give lots of smaller buffers
     */
-    numbuffs = BIGBUFFSIZE/abuf_size;
+    numbuffs = bigbuffsize/abuf_size;
     buffarr = (char**)malloc(numbuffs*sizeof(char*));
     for (i=0; i<numbuffs; i++)
 	buffarr[i] = bigbuff + i * abuf_size;
@@ -102,26 +133,28 @@ void init_shm(void)
 	ErrDie("shmctl");
 
 #if USEBUFFLOCK
+    if (!getuid()) {
 	/* Ok, go root to lock the buffers down */
-    if(setreuid(geteuid(), getuid()) == -1)
-    {
-	fprintf(stderr, "%s: setreuid: %s: continuing anyway\n",
-	    progname, strerror(errno));
-	fprintf(stderr, "real uid = %d, effective uid = %d\n",
-	    getuid(), geteuid());
-    }
-
-    if(shmctl(shmid, SHM_LOCK, NULL) || shmctl(shmid2, SHM_LOCK, NULL))
-	fprintf(stderr,
-	    "%s: shmctl: %s: continuing with unlocked buffers\n",
+	if(setreuid(geteuid(), getuid()) == -1)
+	{
+	    fprintf(stderr, "%s: setreuid: %s: continuing anyway\n",
 		progname, strerror(errno));
+	    fprintf(stderr, "real uid = %d, effective uid = %d\n",
+		getuid(), geteuid());
+	}
 
-    if(setreuid(geteuid(), getuid()) == -1)
-    {
-	fprintf(stderr, "%s: setreuid: %s: continuing anyway\n",
-	    progname, strerror(errno));
-	fprintf(stderr, "real uid = %d, effective uid = %d\n",
-	    getuid(), geteuid());
+	if(shmctl(shmid, SHM_LOCK, NULL) || shmctl(shmid2, SHM_LOCK, NULL))
+	    fprintf(stderr,
+		"%s: shmctl: %s: continuing with unlocked buffers\n",
+		    progname, strerror(errno));
+
+	if(setreuid(geteuid(), getuid()) == -1)
+	{
+	    fprintf(stderr, "%s: setreuid: %s: continuing anyway\n",
+		progname, strerror(errno));
+	    fprintf(stderr, "real uid = %d, effective uid = %d\n",
+		getuid(), geteuid());
+	}
     }
 
 #endif
@@ -179,8 +212,6 @@ void down(int *semblk, int xsemnum)
 void shmrec(int outfd, int totalcount, int terminate)
 {
 
-    pid_t pid;
-
     sync();
 
     pid = fork();
@@ -246,13 +277,13 @@ void shmrec(int outfd, int totalcount, int terminate)
 	    }
 	    /* Update the count for this block */
 	    buffinf[cbuff].count = numrd;
-	    /* Mark the buffer dirty */
-	    up(disksemid, cbuff);
 	    /* Update the amount done */
 	    totalrd += numrd;
 	    /* Tell the reader to stop if needed */
 	    if ((totalrd >= totalcount) && terminate)
 		buffinf[cbuff].last = 1;
+	    /* Mark the buffer dirty */
+	    up(disksemid, cbuff);
 	    /* Update the counter */
 	    cbuff++;
 	    cbuff%=numbuffs;
@@ -315,9 +346,9 @@ void diskread(int infd, int totalplay, char hd_buf[20], int terminate,
 	{
 	    tmppt += count; numread += count;
 	}
-#ifdef DEBUG
-	fprintf(stderr, "in:%d, %d\n", cbuff, numread);
-#endif
+	if (debug >= 2)
+	    fprintf(stderr, "in:%d, %d\n", cbuff, numread);
+
 	/* Update the count for this block */
 	buffinf[cbuff].count = numread;
 	totalread += numread;
@@ -329,9 +360,9 @@ void diskread(int infd, int totalplay, char hd_buf[20], int terminate,
 	else
 	    if(cbuff == numbuffs-1)
 	    {
-#ifdef DEBUG
-fprintf(stderr, "Triggering (in loop)\n");
-#endif
+	      if (debug)
+	            fprintf(stderr, "Triggering (in loop)\n");
+
 		for(i = 0; i < numbuffs; i++)
 		    up(sndsemid,i);
 		    triggered = 1;
@@ -344,20 +375,21 @@ fprintf(stderr, "Triggering (in loop)\n");
     if(terminate)
     {
 	buffinf[cbuff].last = 1;
-	if(!triggered)
-#ifdef DEBUG
-fprintf(stderr, "Triggering (after loop, partial)\n");
-#endif
+	if(!triggered) {
+	    if (debug)
+	        fprintf(stderr, "Triggering (after loop, partial)\n");
+
 	    /* If it wasn't triggered, we haven't filled past cbuff */
 	    for(i = 0; i < cbuff; i++)
 		up(sndsemid, i);
+	}
 	up(sndsemid, cbuff);
     }
     else if((!triggered) && (cbuff == numbuffs-1))
     {
-#ifdef DEBUG
-fprintf(stderr, "Triggering (after loop, full)\n");
-#endif
+        if (debug)
+            fprintf(stderr, "Triggering (after loop, full)\n");
+
 	for(i = 0; i < numbuffs; i++)
 	    up(sndsemid,i);
 	    triggered = 1;
@@ -399,9 +431,9 @@ volatile void audiowrite(void)
 	    numwr += count;
 	    tmpptr += count;
 	}
-#ifdef DEBUG
-fprintf(stderr, "out:%d, %d\n", cbuff, numwr);
-#endif
+	if (debug >= 2)
+            fprintf(stderr, "out:%d, %d\n", cbuff, numwr);
+
 	/* Was it the last buffer? */
 	if (buffinf[cbuff].last)
 	{
@@ -423,13 +455,16 @@ fprintf(stderr, "out:%d, %d\n", cbuff, numwr);
 void initsems(int disks, int snds)
 {
     int i,j;
+    union semun dsu, ssu;
 
+    dsu.val = disks;
+    ssu.val = snds;
     for (i=0;i<numsemblks;i++)
 	for (j=0; j<SEMMSL;j++)
 	{
-	    if(semctl(disksemid[i], j, SETVAL, disks) == -1)
+	    if(semctl(disksemid[i], j, SETVAL, dsu) == -1)
 		ErrDie("semctl");
-	    if(semctl(sndsemid[i], j, SETVAL, snds) == -1)
+	    if(semctl(sndsemid[i], j, SETVAL, ssu) == -1)
 		ErrDie("semctl");
 	}
 }
@@ -437,17 +472,25 @@ void initsems(int disks, int snds)
 void cleanupsems(void)
 {
     int i;
+    union semun s;
+
+    s.val = 0;
 
     for (i = 0; i < numsemblks; i++)
     {
-	semctl(disksemid[i], 0, IPC_RMID, 0);
-	semctl(sndsemid[i], 0, IPC_RMID, 0);
+	semctl(disksemid[i], 0, IPC_RMID, s);
+	semctl(sndsemid[i], 0, IPC_RMID, s);
     }
 }
 
 static void sighandler(int i)
 {
-    fprintf(stderr, "signal %d received, cleaning up.\n", i);
+    if (debug)
+        fprintf(stderr, "signal %d received, cleaning up.\n", i);
+
+    if (!recorder && pid)
+        kill(pid,9);
+    cleanup_audio();
     cleanupsems();
     exit(1);
 }
