@@ -12,17 +12,21 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/soundcard.h>
+#include <sys/ioctl.h>
 
 #define AUDIO "/dev/dsp"
 
 /* Globals */
 int audio, abuf_size, fmt_mask;
+char *audev = AUDIO;
 
 /* Prototypes */
 void sync_audio(void);
+void cleanup_audio(void);
 
 /* Extern globals */
 extern char *progname;
+extern int debug;
 
 /* Extern prototypes */
 extern void ErrDie(char *err);
@@ -30,19 +34,25 @@ extern void Die(char *err);
 
 void init_sound(int recorder)
 {
+
     /* Attempt to open the audio device */
-    audio = open(AUDIO, (recorder)? O_RDONLY : O_WRONLY);
+    audio = open(audev, (recorder)? O_RDONLY : O_WRONLY);
     if (audio == -1)
-	ErrDie(AUDIO);
+	ErrDie(audev);
+#if 1
+    if (ioctl(audio, SNDCTL_DSP_GETBLKSIZE, &abuf_size) < 0) ErrDie(audev);
+    if (debug) 
+        fprintf(stderr, "abuf_size = %d\n", abuf_size);
 #if 0
-    if (ioctl(audio, SNDCTL_DSP_GETBLKSIZE, &abuf_size) < 0) ErrDie(AUDIO);
-    if (abuf_size < 4096 || abuf_size > 65536) Die("invalid audio buffer size");
-    fprintf(stderr, "abuf_size = %d\n", abuf_size);
+    if (abuf_size < 1024 || abuf_size > 65536) Die("invalid audio buffer size");
+#else
+    if (abuf_size < 4096) abuf_size = 4096; /* Seems reasonable */
+#endif
 #else
     abuf_size = 65536;
 #endif
 #if 1
-    if (ioctl(audio, SNDCTL_DSP_GETFMTS, &fmt_mask) < 0) ErrDie(AUDIO);
+    if (ioctl(audio, SNDCTL_DSP_GETFMTS, &fmt_mask) < 0) ErrDie(audev);
 #endif
 }
 
@@ -60,11 +70,11 @@ void snd_parm(int speed, int bits, int stereo)
 
 	/* Set the sample speed, size and stereoness */
 	if (ioctl(audio, SNDCTL_DSP_SAMPLESIZE, &bits) < 0)
-	    ErrDie(AUDIO);
+	    ErrDie(audev);
 	if (ioctl(audio, SNDCTL_DSP_STEREO, &stereo) < 0)
-	    ErrDie(AUDIO);
+	    ErrDie(audev);
 	if (ioctl(audio, SNDCTL_DSP_SPEED, &speed) < 0)
-	    ErrDie(AUDIO);
+	    ErrDie(audev);
     }
     oldspeed = speed; oldbits = bits; oldstereo = stereo;
 }
@@ -72,5 +82,15 @@ void snd_parm(int speed, int bits, int stereo)
 void sync_audio(void)
 {
     if (ioctl (audio, SNDCTL_DSP_SYNC, NULL) < 0)
-	ErrDie(AUDIO);
+	ErrDie(audev);
 }
+
+void cleanup_audio(void)
+{
+    if (ioctl (audio, SNDCTL_DSP_RESET) < 0)
+	ErrDie(audev);
+    close(audio);
+}
+
+
+
